#include <lpfgall.h>
#include <math.h>

// defines pertinent to computation
const float INFTY = 1e11;        // very large number
const float EPSILON = 1.0/INFTY; // very small number different from zero
const float DT = 0.1;            // time step (days)
const float MAXERROR = 1e-5;     // error threshold for evaluating carbon flow
const float VIS_STEP = 1.0;      // when to visualise model (days)

/* -------------------------------------------------------------------------- */
// defines pertinent to kiwifruit development (values from Cieslak et al. (2010))

// defines pertinent to kiwifruit cane
const float MTNCE_CANE = 0.00184;    // cane maintenance coefficient (g C / g C day)
const float CANE_ORDER = 0;

// defines pertinent to kiwifruit shoot
const float MAX_NODES = 60.0;    // maximum number of nodes on a shoot
const float MAX_GROWTH = 240.0;  // max growing days after budbreak (days)
const float AX_SHOOT_ORDER = CANE_ORDER+1;
const float REGROWTH_VIGOUR = 1.0;
const int NUM_PREFORMED_NODES = 7; // the number of preformed metamers in this season's axillary bud

// defines pertinent to leaves
const float TAU_LEAF = 5.53;            // size of leaf 'growth window' (days)
const float LEAF_DENSITY = 0.00819;     // biomass to area (g DW / cm^2)
const float LEAF_BIO2CARB = 0.45;       // biomass to carbon (g C / g DW)
const float MTNCE_LEAF = 0.00806;       // leaf maintenace respiration co-efficient (g C / g C day)
const float INCLINATION_ANGLE = 33.0;   // a leaf's inclination angle
const float PHYLLOTAXIS = 144.0;        // the phyllotactic angle
const float LEAF_LOADING = 1.0;         // rate of carbon loading (dimensionless)
const float LEAF_INITIAL_CARBON = 0.1*LEAF_DENSITY*LEAF_BIO2CARB; // initial carbon in a leaf (g C) at time of appearance

// defines pertinent to internodes
const float TAU_INDE = TAU_LEAF*0.5;      // size of internode 'growth window' (days)
const float INTERNODE_DENSITY = 0.189429; // carbon to volume (g C / cm^3)
const float MTNCE_INDE = 0.00527;         // internode maintenace respiration co-efficient (g C / g C day)
const float INDE_TIMING_EXPANSION = 6.0;  // inflection point for elongation of internodes (day)
const float SEC_GROWTH_RATE = 0.003;      // coefficient of proportionality for secondary growth (dimensionless)
const float SEC_LOGISTIC_RATE = 0.28;     // logisitic growth rate for young internodes (dimensionless)
const float INDE_RESERVES = 0.0592596;    // ratio of carbon reserves to structural carbon for mature internodes
const float INITIAL_INDE_RADIUS = 0.15;   // initial internode radius (cm)

// defines pertinent to apex
const float PHYLLOCHRON = 2.9;       // age at which the apex produces a metamer
const float CARBON_THRESHOLD = 0.01; // carbon concentration threshold for production of new metamers (g C / cm^3 ?)

// defines pertinent to flowers and fruit
const float MAX_FLOWER_AGE = 60.0;         // days for flower to fully open (or accumulate INITIAL_FRUIT_CARBON)
const float FLOWER_FINAL_DIAMETER = 6.8;   // maximum final diameter of a flower (cm)
const float FLOWER_DENSITY = 6.8 / 0.032;  // cm per g C in flower (this is just for visualisation)
const float INITIAL_FLOWER_CARBON = 0.001; // initial carbon in a flower

const float MAX_FRUIT_AGE = 180.0; // days before fruit harvest (after anthesis)
const float FRUIT_DENSITY = 3.5;   // cm^3 per g C in fruit (this is just for visualisation)
const float MTNCE_FRUIT = 0.00143; // fruit maintenace respiration co-efficient (g C / g C day)
const float MAX_FRUIT_DW = 32.3;   // maximum fruit dry weight at saturation (g DW) (See Minchin et al (2009) and Snelgar et al (1986))
const float MAX_FRUIT_CARBON = MAX_FRUIT_DW * 0.45; // maximum DW * g C / g DW
const float INITIAL_FRUIT_CARBON = 0.032; // initial carbon in fruit
/* -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- */
// defines pertinent to auxin transport (values from Prusinkiewicz et al. (2009))

// auxin transport
const float T =	0.5;  // polar transport coefficient: default 0.5, reduce to 0.2 in axr3
const float D =	0.02; // diffusion coefficient

// PIN allocation
const float rho = 10.0;	  // PIN allocation rate; default 10; reduce to 5 in tir3 and max,tir3
const float n =	3.0f;	  // Hill exponent
const float K =	0.5f;	  // Hill saturation constant
const float rho_0 = 0.15; // base PIN production
const float mu = 7.5;	  // PIN turnover: default 7.5; reduce to 2.5 in max and max,tir3
const float K_power = pow(K, n);

// auxin production and turnover
const float H =	10;        // target auxin concentration at the source - increase to 14 in axr1
const float H_r = 2.5;     // target auxin concentration after flowering: default 2.5; 10 for axr3
const float sigma = 10;    // auxin production rate
const float sigma_r = 0.5; // residual auxin production: default 10, decrease to 0.5 for basipetal 
const float nu = 0.005;	   // auxin decay rate: default 0.005, 0.03 for convergent; 0.2 for acropetal
const float nu_root = 1.0; // auxin turnover in the root

const float FLUXTH = 6.7;   // threshold auxin flux that activates lateral apex
const float PIN_INIT = 0.5; // initial PIN concentration in a new metamer 
			    // of a given axis (i.e., excluding lateral bud) 

#define InnatePolarity	1	// 0 - polarity OFF (rho j->i = rho i->j)
				// 1 - polarity ON  (rho j->i = 0)
#define InitLatAuxinH	0	// initial auxin concentration in lateral bud:
				// 0 - low (=0), 1 - high (equal to parent) 
/* -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- */
// defines pertinent to Biomechanics (values from Prusinkiewicz et al. (2007b))

const float BM_KAPPA = 5.0e7; // rotational spring constant
const float BM_MAXERROR = 0.01;  // maximum error in equilibrium
const float BM_RELAX = 0.5;      // relaxation coefficient

const V3f Gravity(0, -9.81, 0);	// the gravity vector

/* Rotation of a 3D vector a in the xy plane by angle alpha */
V3f VecRotate(V3f a, float alpha)
{
	V3f c;
	c.x = a.x * cos(alpha) - a.y * sin(alpha);
	c.y = a.x * sin(alpha) + a.y * cos(alpha);
	c.z = 0;
	return c;
}
/* -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- */
// Definition of functions and structures for C-TRAM model

#include "functions.h"

// Declaration of the structure instantaneously representing a linearized organ
struct LinearData
{
   float e; // EMF
   float r; // resistance
};

// Function for connecting circuits in parallel
LinearData in_parallel(LinearData c1, LinearData c2)
{
  LinearData c;
  c.e = (c1.e * c2.r + c2.e * c1.r) / (c1.r + c2.r);
  c.r = (c1.r * c2.r) / (c1.r + c2.r);
  return c;
}

// Declaration of the structure that characterizes a sink/source
// in the paper, the members were renamed so that no reference to electric circuits was needed
// in the code, these names are not changed from the implementation of Prusinkiewicz et al. (2007)
struct SegmentData
{
  LinearData cp;  // parallel part of the circuit: EMF and res. to ground
  float rs;	  // resistance in series
  LinearData ceq; // Equivalent circuit: resultant EMF and internal resistance
  float i;	  // current through the resistance rs
  float v;	  // potential at the (right) node
  float err;	  // current difference between iteratations
  float q;	  // accumulated charge in segment
  void update_charge();
};

// function to update the carbon in sink/source depending on carbon flow
void SegmentData::update_charge()
{
  q += i * DT;
  if (q < 0.0)
    q = 0.0;
  return;
}

SegmentData InitSegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0};
/* -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- */
// Declaration of structure used for auxin transport
struct AuxinData
{
  float c;	// auxin concentration
  // function to update production of auxin
  void update_c (float dt);
};
void AuxinData::update_c (float dt)
{
  c += sigma*(H-c)*DT;
}
AuxinData metamer_init_state = {0};

struct WallData
{
  float flux;	// flux (from right to left)
  float PINi;	// PIN concentration in membrane i->j (right->left)
  float PINj;	// PIN concentration in membrane j->i (left->right)
};
WallData wall_init_state = {0,0,0};
/* -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- */
// Declaration of structure used for biomechanics data
struct BiomechanicsData
{
  float s;		// internode length
  float mass;		// mass of a node
  float inv_ke;		// elasticity at a node (stored as its inverse)
  float sigma_mass;	// combined mass to the right of a node
  float torque;		// combined torque
  float ba;		// branching angle
  V3f P;		// left node position
  V3f H;		// orientation of the internode

  // constructor to initialise the data
  BiomechanicsData (void) :
    s(0), mass(0), inv_ke(0), sigma_mass(0), torque(0), ba(0), P(0,0,0), H(0,0,0) {}

  // function to set elasticity of an internode based on its attributes
  void update_ke (float radius, float age, float node);
};

void BiomechanicsData::update_ke (float radius, float age, float node)
{
  float ke_age = BM_KAPPA;

  // the tip of a long kiwifruit shoot usually droops,
  // so, here, we set the elasticity for long shoots based on internode age
  if (node >= 18)
  {
    ke_age = (BM_KAPPA - BM_KAPPA*0.001) *
             exp((age-40.0)*0.3) / (1.0 + exp((age-40.0)*0.3)) +
             BM_KAPPA*0.001;
  }

  // set elasticity based on radius
  float ke = ke_age * pow(radius,4.0f) * M_PI * 0.5;

  // update the value in the data structure
  inv_ke = 1.0 / ke;
}
/* -------------------------------------------------------------------------- */


// Enumeration of computation phases
#define Develop         1
#define AuxinFluxes	2  // two phases for auxin transport
#define AuxinTransport  3
#define Linearise       4  // five phases for carbon dynamics 
#define Fold            5
#define Unfold 	        6
#define UpdateFlow      7
#define Debug           8
#define RelaxFold	9  // two phases for biomechanics
#define RelaxUnfold	10


// Declarations of global variables 
int Phase;          // the current computation phase
float Serror;       // the error in computation for carbon dynamics, measuring convergence to the solution
int error_count;    // the number of times 'error' is above MAXERROR
float BMerror;       // the error in computatino for biomechanics
float sim_time;     // the current simulation time (steps of DT)
float vis_time;     // the time to visualise the model

// Parameters to be fitted: mainly the sink priorities
float Q_LEAF_PRI = 0.01;      // for primary leaf growth
float Q_LEAF_MAN = 0.00001;   // for leaf maintenance respiration
float Q_INDE_PRI = 0.01;      // for primary internode growth
float Q_INDE_SEC = 0.05;      // for secondary internode growth
float Q_INDE_MAN = 0.00001;   // for internode maintenance respiration
float Q_RESERVES = 0.1;       // for starch synthesis in internodes
float Q_FLOWER = 0.01;        // for flower growth
float Q_FRUIT_PRI = 10.0;     // for fruit growth
float Q_FRUIT_MAN = 0.00001;  // for fruit maintenance
float Q_ROOTS = 30.0;         // for root growth
float Q_ROOTS_MAN = 0.00001;  // for root maintenance
float Q_ROOTS_RESERVES = 0.1; // for root reserves

float Q_LEAF_SRC = 10.0;     // coefficient for carbon supply from leaf
float Q_INDE_SRC = 10.0;     // coefficient for carbon supply from reserves in internodes
float Q_ROOTS_SRC = 10.0;    // coefficient for carbon supply from reserves in roots

float INDE_SYNTHESIS = 0.1;   // rate of starch synthesis based on structural carbon
float INDE_HYDROLYSIS = 1.0;  // rate of starch hydrolysis based on structural carbon (must be <= 1)
float ROOTS_SYNTHESIS = 0.1;  // rate of starch synthesis based on structural carbon
float ROOTS_HYDROLYSIS = 1.0; // rate of starch hydrolysis based on structural carbon (must be <= 1)

float TRANSPORT_RS = 0.025; // coefficient of resistance along transport pathway (g C / cm^4 s)


// Initialization of global variables
Start:
{
  Phase = Develop;
  Serror = 0.0;
  error_count = 0;
  BMerror = 0.0;
  sim_time = 0.0;
  vis_time = 0.0;

  // set seed for random number generator
  sran(123456);

  UseView(KiwifruitShoot);
  UseView(CarbonDynamics);
  UseView(AuxinFlow);
  UseView(Biomechanics);
}

StartEach:
{
  UseGroup(Phase);
  switch (Phase)
  {
    case Develop: // growth and development phase
      Forward();
      if (sim_time < MAX_GROWTH)
      {
        sim_time += DT;
      }
      else
      {
        Printf ("End of season!\n");
        Stop(); // stop simulation at end of season 
      }
      break;
    case AuxinFluxes: // calculate auxin fluxes
      Forward();
      break;
    case AuxinTransport: // auxin from apex transported basipetally
      Backward();
      break;
    case Linearise: // set carbon demand (linearise source/sink equations)
      Forward();
      break;
    case Fold: // left pass - accumulation of the resultant E and R
      Backward();
      break;
    case Unfold: // right pass - calculation of carbon flows (and error)
      Forward();
      break;  
    case UpdateFlow: // calculate carbon flow into/from sink/source (and check error level)
      Forward();
      Serror = 0;
      break;
    case Debug: // prints out info of all sinks/sources with error > MAX_ERROR
      Printf ("Warning: error > MAXERROR (%g > %g) after %d iterations\n", Serror, MAXERROR, error_count);
      Printf ("Dumping 'S' modules at sim_time %g:\n", sim_time);
      Forward();
      break;
    case RelaxFold:
      Backward();
      break;
    case RelaxUnfold:
      Forward();
      BMerror = 0.0;
      break;
  }
}

EndEach:
{   
  switch (Phase) // the control automaton determines next phase type
  {
    case Develop:
      if (sim_time < MAX_GROWTH)
      {
        Phase = AuxinFluxes;
        if (sim_time >= vis_time)
        {
          DisplayFrame();
          vis_time += VIS_STEP;
        }
        error_count = 0;
      }
      break;
    case AuxinFluxes:
      Phase = AuxinTransport;
      break;
    case AuxinTransport:
      Phase = Linearise;
      break;
    case Linearise:
      Phase = Fold;
      break;
    case Fold:
      Phase = Unfold;
      break;
    case Unfold:
      Phase = UpdateFlow;
      break;
    case UpdateFlow:
      if (Serror < MAXERROR)
      {
	Phase = RelaxFold;
        error_count = 0;
      }
      else
      {
        if (error_count > 1000)
        {
          Phase = Debug;
        }
        else
        {
          ++error_count;
	  Phase = Linearise;
        }
      }
      break;
    case Debug:
      Stop();
      break;
    case RelaxFold:
      Phase = RelaxUnfold;
      break;
    case RelaxUnfold:
      if (BMerror > BM_MAXERROR)
        Phase = RelaxFold;
      else
        Phase = Develop;
      break;
  }
}

/* -------------------------------------------------------------------------- */
// module type declarations
module S(SegmentData);                                 // a sink/source component
module A(AuxinData);                                   // a auxin transport component
module W(WallData);                                    // a wall auxin transport component
module B(BiomechanicsData);                            // a biomechanics component

module Internode(float,float,float,float,float,float); // internode with attributes: node,age,order,developmental age,length,radius
module Leaf(float,float,float,float,float);            // leaf with attributes: node,age,order,developmental age,area
module Flower(float,float,float);                      // flower with attributes: node,age,diameter
module Fruit(float,float,float);                       // fruit with attributes: node,age,volume
module Apex(float,float,float,float,float);            // apex with attributes: node,age,physiological age,vigour,order
module AxBud(float,float,float,float);                 // axillary bud with attributes: node, age, vigour, order. It produces flower buds (first 7 nodes) and a regrowth if the shoot is pruned.
module ProduceMetamer(float,float,AuxinData);          // decomposed into a new metamer: internode, leaf, and possibly fruit
module Cane();                                         // the cane internode that the shoot is attached to
module Prune();                                        // handles a pruning event in the Develop phase
/* -------------------------------------------------------------------------- */

derivation length: 999999;

Axiom: SB() Right(45) f(250) EB() // adjust the initial view
       Right(110) GetHead(0,0,0) Cane();


/* Development Rules -------------------------------------------------------- */

group Develop: // production of new metamers, growth of existing organs, and addition of carbon by photosynthesis

Apex(node,age,physio_age,vigour,order) S(sd_apex) W(d) A(s):
{
  age += DT;
  physio_age += ApexRateofAppearance() * DT;
  sd_apex.update_charge();

  // auxin production in apex (assuming DT is same for auxin transport aspect)
  s.update_c(DT);

  if (node <= MAX_NODES && vigour > 0.0)
  {
    if (physio_age >= 1.0)
    {
      if (ApexGrowthCessation(sd_apex.v, vigour, node))
        produce Apex(node,age,physio_age,0,order) S(sd_apex) W(d) A(s); // set vigour to zero; apex aborts

      // else produce a metamer and reset apex's physio_age to zero

      // initialize state of wall between metamers
      WallData new_wall_state = wall_init_state;
      new_wall_state.PINi = PIN_INIT;

      produce GetPos(0,0,0) GetHead(0,0,0) ProduceMetamer(node,order,s)
              Apex(node+1,age,0.0,vigour,order) S(sd_apex) W(new_wall_state) A(s);
    }
  }
  // Apex is either waiting (phyllochron not reached) or has aborted (vigour=0)
  produce Apex(node,age,physio_age,vigour,order) S(sd_apex) W(d) A(s);
}

Internode(node,age,order,dev_age,length,radius) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis) W(d) A(s) B(bm):
{
  age += DT;
  dev_age += IndeSpecificGrowthRate() * DT;

  // NOTE: carbon is not accumulated for maintenance respiration
  sd_maintenance.q = sd_maintenance.i * DT;

  // now, test if minimum is met and start senescence for eventual abscission
  if (fabs(sd_maintenance.q - (IndeMaintenance (sd_primary.q,order) * DT)) > MAXERROR)
  {
    if (sim_time > DT) // ignore for first time step
    {
      Printf ("Abscission INDE(o:%g,n:%g): %g > %g\n", order, node, fabs(sd_maintenance.q - (IndeMaintenance (sd_primary.q,order) * DT)), MAXERROR);
      produce Internode(node,age,order,dev_age,length,radius) S(InitSegmentData) S(InitSegmentData) S(InitSegmentData) S(InitSegmentData) W(d) A(s) B(bm);
    }
  }

  // accumulate carbon for primary growth
  sd_primary.update_charge();

  length += SinkUnloading(sd_primary.v, Q_INDE_PRI) *
            IndePrimaryGrowthRate(dev_age, length) * DT;

  radius += SinkUnloading(sd_primary.v, Q_INDE_SEC) *
            IndeSecondaryGrowthRate(dev_age, radius) * DT;

  // for reserves
  sd_reserves.update_charge();
  sd_reserves.q += sd_hydrolysis.i * DT; // some stored carbon is lost to hydrolysis
  if (sd_reserves.q < 0.0)
    sd_reserves.q = 0.0;

  // set resistance along transport pathway (unless it has been girdled)
  if (sd_primary.rs < INFTY)
    sd_primary.rs = IndeFlowResistance(length, radius, TRANSPORT_RS);

  // update biomechanics
  bm.s = length;
  bm.mass = sd_primary.q;
  bm.update_ke(radius,age,node);

  produce Internode(node,age,order,dev_age,length,radius) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis) W(d) A(s) B(bm);
}

Leaf(node,age,order,dev_age,area) S(sd_primary) S(sd_maintenance) S(sd_source):
{
  age += DT;
  dev_age += LeafSpecificGrowthRate() * DT;

  // NOTE: carbon is not accumulated for maintenance respiration
  sd_maintenance.q = sd_maintenance.i * DT;

  // now, test if minimum is met and start senescence for eventual abscission
  if (fabs(sd_maintenance.q - (LeafMaintenance (sd_primary.q) * DT)) > MAXERROR)
  {
    Printf ("Abscission LEAF(%g): %g > %g\n", node, fabs(sd_maintenance.q - (LeafMaintenance (sd_primary.q) * DT)), MAXERROR);
    produce Leaf(node,age,order,dev_age,area) S(InitSegmentData) S(InitSegmentData) S(InitSegmentData);
  }

  sd_primary.update_charge();

  sd_source.update_charge();

  // photosynthesis - this could be improved by radiance estimate and light reponse curve 
  sd_source.q += LeafCarbonFromPhoto (sd_source.q, sd_primary.q, 0.35, area) * DT;

  produce Leaf(node,age,order,dev_age,area) S(sd_primary) S(sd_maintenance) S(sd_source);
}

AxBud(node,age,vigour,order) S(sd_apex) W(d) A(s):
{
  age += DT;
  sd_apex.update_charge();

  // production of auxin
  s.update_c(DT);

  if (order == CANE_ORDER)
  {
    // set initial "auxin" state of preformed metamers
    AuxinData m_init_state = metamer_init_state;
    m_init_state.c = H;

    int node;
    for (node = 1; node <= NUM_PREFORMED_NODES; node++)
    {
      nproduce GetPos(0,0,0) GetHead(0,0,0) // used by ProduceMetamer to set biomechanics
               ProduceMetamer(node,AX_SHOOT_ORDER,m_init_state); // with order=AX_SHOOT_ORDER
    }
    // vigour set to one
    produce Apex(node,0,0,1,AX_SHOOT_ORDER) S(InitSegmentData) W(wall_init_state) A(m_init_state);  
  }
  else if (order == AX_SHOOT_ORDER && node <= 8 && vigour > 0)
  {
    // possibly, produce a flower
    if (FlowerDevelopment(node))
    {
      SegmentData sd_flower_primary = InitSegmentData;
      sd_flower_primary.q = INITIAL_FLOWER_CARBON;
      produce Flower(node,0,INITIAL_FLOWER_CARBON*FLOWER_DENSITY) S(sd_flower_primary);
    }
    else // the axillary bud aborts (does not produce flower)
      produce AxBud(node,age,0,order) S(sd_apex) W(d) A(s);
  }
  else // a regrowth may occur: Apex with node=1,age=0,physio_age=0,vigour=1,order=axbud's order+1
  {
    if (d.flux > FLUXTH)
      produce Down(INCLINATION_ANGLE) Apex(1,0,0,REGROWTH_VIGOUR,order+1) S(InitSegmentData) W(d) A(s);
    else
      produce AxBud(node,age,vigour,order) S(sd_apex) W(d) A(s);
  }
}

Flower(node,age,diameter) S(sd_primary):
{
  age += DT;
  sd_primary.update_charge();

  // the flower grows for some given amount of time
  if (age >= MAX_FLOWER_AGE)
  {
    SegmentData sd_fruit_primary = InitSegmentData;
    // if carbon is not limited, the flower should have INITIAL_FRUIT_CARBON accumulated
    sd_fruit_primary.q = sd_primary.q;

    SegmentData sd_maintenance = InitSegmentData;
    sd_maintenance.q = FruitMaintenance(sd_primary.q) * DT;

    // at this stage, it should produce fruit
    produce Fruit(node,0,0) S(sd_fruit_primary) S(sd_maintenance);
  }

  produce Flower(node,age,diameter) S(sd_primary);
}

Fruit(node,age,volume) S(sd_primary) S(sd_maintenance):
{
  age += DT;

  // NOTE: carbon is not accumulated for maintenance respiration
  sd_maintenance.q = sd_maintenance.i * DT;

  // now, test if minimum is met and start senescence for eventual abscission
  if (fabs(sd_maintenance.q - (FruitMaintenance (sd_primary.q) * DT)) > MAXERROR)
  {
    Printf ("Abscission FRUIT(%g): %g > %g\n", node, fabs(sd_maintenance.q - (FruitMaintenance (sd_primary.q) * DT)), MAXERROR);
    produce Fruit(node,age,volume) S(InitSegmentData) S(InitSegmentData);
  }

  // update fruit carbon (size)
  sd_primary.update_charge();

  produce Fruit(node,age,volume) S(sd_primary) S(sd_maintenance);
}

S(sd):
{
  // if r == INFTY this is an inert S between branches
  if (sd.cp.r != INFTY)
    Printf ("PANIC! Orphan S() module found. Check 'Develop' phase.\n");  
  produce S(sd);
}


GetPos(x,y,z) GetHead(hx,hy,hz) ProduceMetamer(node,order,s):
{
  // set initial conditions for internode
  SegmentData sd_internode_primary = InitSegmentData;
  float radius = INITIAL_INDE_RADIUS;
  float length = IndeInitialLength(node,order);
  sd_internode_primary.q = INTERNODE_DENSITY * M_PI * radius * radius * length;
  float inde_age = IndeInitialDevAge();

  SegmentData sd_maintenance = InitSegmentData;
  sd_maintenance.q = IndeMaintenance(sd_internode_primary.q,order) * DT;

  SegmentData sd_internode_reserves = InitSegmentData;
  sd_internode_reserves.q = 0.001 * sd_internode_primary.q;

  // set the resistance along transport pathway
  // (in the first S module associated with Internode)
  sd_internode_primary.rs = IndeFlowResistance(length, radius, TRANSPORT_RS);

  // set initial conditions for leaf
  SegmentData sd_leaf_primary = InitSegmentData;
  sd_leaf_primary.q = LEAF_INITIAL_CARBON;
  float leaf_age = LeafInitialDevAge(node,order);
  float area = Carb2Area(sd_leaf_primary.q);

  // set initial conditions for axillary bud
  float axbud_vigour;
  if (node <= 8 && order == AX_SHOOT_ORDER)
    axbud_vigour = 1.0;
  else 
    axbud_vigour = 0.0;

  // initialize auxin transport
  // (a) initialize auxin concentration of new metamer
  AuxinData m_init_state = metamer_init_state;
  m_init_state.c = s.c;

  // (b) initialize auxin concentration of lateral
  AuxinData l_apex_init_state = metamer_init_state;
  if (InitLatAuxinH==0)
    l_apex_init_state.c = 0; 
  else
    l_apex_init_state.c = s.c;

  // (c) initialize state of wall between metamers
  WallData new_wall_state = wall_init_state;
  new_wall_state.PINi = PIN_INIT;

  // initialize biomechanics
  BiomechanicsData bm;
  bm.s = length;
  bm.mass = sd_internode_primary.q;
  bm.update_ke(radius,0,node);
  bm.H = V3f(hx,hy,hz);
  bm.P = V3f(x,y,z) + bm.s*bm.H;

  // set branching angle depending on shoot order
  if (node == 1)
  {
    if (order == AX_SHOOT_ORDER)
      bm.ba = M_PI*0.5;
    else
      bm.ba = -INCLINATION_ANGLE/180.0 * M_PI;
  }
  else
    bm.ba = 0.0;

  // produce an internode, a leaf and an axillary bud
  produce RollL(PHYLLOTAXIS)
          Internode(node,0,order,inde_age,length,radius) S(sd_internode_primary)
                                                         S(sd_maintenance)
                                                         S(sd_internode_reserves)
                                                         S(InitSegmentData)
                                                         W(new_wall_state) A(m_init_state)
                                                         B(bm)
          SB() Leaf(node,0,order,leaf_age,area) S(sd_leaf_primary)
                                                S(InitSegmentData)
                                                S(InitSegmentData)
          EB()
          // Careful! C-TRAM rules cannot handle more than one branching-point: an inert sink is needed between branches
          S(InitSegmentData)
          SB() RollR(PHYLLOTAXIS*node) // FOR 2D BIOMECHANICS ONLY: unravel the phyllotaxis so laterals are in same plane as parent shoot
               AxBud(node,0,axbud_vigour,order) S(InitSegmentData)
                                                W(wall_init_state) A(l_apex_init_state)
          EB()
          ;
}

GetHead(hx,hy,hz) Cane():
{
  // the section of cane is modelled as one internode
  SegmentData sd_internode_primary = InitSegmentData;
  SegmentData sd_internode_reserves = InitSegmentData;
  SegmentData sd_maintenance = InitSegmentData;

  float age = 365;
  float dev_age = 365;
  float radius = 0.5;
  float length = 10.0;

  sd_internode_primary.q = INTERNODE_DENSITY * M_PI * radius * radius * length;
  sd_maintenance.q = IndeMaintenance(sd_internode_primary.q,CANE_ORDER) * DT;

  sd_internode_reserves.q = CaneIndeInitialReserves (sd_internode_primary.q);
  // check for numerical error (maybe 'float' isn't best data type to use)
  if (sd_internode_primary.q*INDE_RESERVES < sd_internode_reserves.q)
    sd_internode_reserves.q = sd_internode_primary.q * INDE_RESERVES*0.99999;

  // set the resistance along transport pathway (in the first S module associated with Internode)
  sd_internode_primary.rs = IndeFlowResistance(length, radius, TRANSPORT_RS);

  // init biomechanics for the section of cane (an internode)
  BiomechanicsData bm;
  bm.s = length;
  bm.mass = sd_internode_primary.q;
  bm.update_ke(radius,age,1);
  bm.ba = 0.0;
  bm.H = V3f(hx,hy,hz);
  bm.P = bm.s*bm.H;

  produce A(metamer_init_state) W(wall_init_state) A(metamer_init_state) // the large auxin sink at the start of shoot
          Internode(1,age,CANE_ORDER,dev_age,length,radius)
            S(sd_internode_primary)
            S(sd_maintenance)
            S(sd_internode_reserves)
            S(InitSegmentData)
            W(wall_init_state) A(metamer_init_state)
            B(bm)
            SB() /* Leaf */ EB() // to ensure consistency for rule matching, place SB() EB() here, where a leaf would have been.
            S(InitSegmentData)   // C-TRAM rules cannot handle more than one branching-point: an inert sink is needed between branches
            SB() Left(90) RollL(90) AxBud(1,age,1,CANE_ORDER) S(InitSegmentData) W(wall_init_state) A(metamer_init_state) EB()
          ;
}

Prune():
{
  // if the shoot was pruned, remove anything above but within the branch
  produce Cut();
}

/* -------------------------------------------------------------------------- */

/* Auxin Transport Rules ---------------------------------------------------- */

group AuxinFluxes: // Calculate fluxes through the wall according to the canalization model

consider: A;

production:

A(sl) < W(w) > A(sr) :	// Auxin flux through membrane, PIN allocation
{
	w.flux = T*sr.c*w.PINi - T*sl.c*w.PINj + D*(sr.c-sl.c);	// Equation 1
	float flux_power;
	if (w.flux >= 0) {	// Equations 2a, 3b
		flux_power = pow(w.flux, n);
		w.PINi += (rho*flux_power/(K_power+flux_power) + rho_0 - mu*w.PINi) * DT;
		w.PINj += (rho_0 - mu*w.PINj)*DT;
	}
	if (w.flux < 0) {	// Equation 2b, 3a with flux i->j = - flux j->i
		flux_power = pow(-w.flux, n);
		w.PINi += (rho_0 - mu*w.PINi) * DT;
		if (InnatePolarity==0)	// use directly Equation 3a
			w.PINj += (rho*flux_power/(K_power+flux_power) + rho_0 - mu*w.PINj) * DT;
		else			// assume rho j->i = 0 due to innate polarity
			w.PINj += (rho_0 - mu*w.PINj)*DT;
	}
	
	produce W(w);
}


/* -------------------------------------------------------------------------- */

group AuxinTransport: // basipetal transport of auxin

consider: W;

production:

W(wl) < A(s) > SB() EB() SB() W(wr) EB() W(wb) : // metamer: lateral branch present
{
	s.c += (wr.flux+wb.flux-wl.flux		// auxin flows in, out,...
		-nu*s.c)*DT;			// ... and decays
	produce A(s);
}

W(wl) < A(s) > W(wr) :				// metamer: lateral branch absent
{
	s.c += (wr.flux-wl.flux-nu*s.c)*DT;	// auxin flows in, out, and decays
	produce A(s);
}

W(wl) < A(s) > SB() EB() SB() W(wr) EB() :	// metamer: main branch absent
{
	s.c += (wr.flux-wl.flux-nu*s.c)*DT;	// auxin flows in, out, and decays
	produce A(s);
}

A(s) > W(wr) :					// root
{
	s.c += (wr.flux-nu_root*s.c)*DT;	// auxin flows in and decays		
	produce A(s);
}

W(wl) < A(s)  :					// apex
{	
	s.c -= (wl.flux + nu*s.c)*DT ;		// auxin flows out and decays			

	// production of auxin should not be done here:
	// if lateral buds are removed and then the apex, the metamer below
	// the pruning cut will start to produce auxin!
        //s.c += sigma*(H-s.c)*DT;		// produce/maintain high auxin concentration

	produce A(s);
}

/* -------------------------------------------------------------------------- */

/* Carbon Dynamics Rules ---------------------------------------------------- */

group Linearise: // linearise equations for sinks/sources

Apex(node,age,physio_age,vigour,order) S(sd_apex):
{
  // apex does not accumulate carbon
  sd_apex.cp.r = INFTY;
  sd_apex.cp.e = 0.0;
  produce Apex(node,age,physio_age,vigour,order) S(sd_apex);
}

Internode(node,age,order,dev_age,length,radius) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis):
{
  float demand; // the sink's demand for carbon, linearized! (conductance in electrical circuit)

  // for internode elongation and thickening, demand depends on unloading and relative growth rate
  demand = M_PI * radius * radius * INTERNODE_DENSITY *
           dSinkUnloading_dc(sd_primary.v, Q_INDE_PRI) *
           IndePrimaryGrowthRate(dev_age ,length)
           +
           2.0 * M_PI * radius * length * INTERNODE_DENSITY *
           dSinkUnloading_dc(sd_primary.v, Q_INDE_SEC) *
           IndeSecondaryGrowthRate(dev_age, radius);

  sd_primary.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_primary.cp.e = sd_primary.v - sd_primary.cp.r * (
                    M_PI * radius * radius * INTERNODE_DENSITY *
                    SinkUnloading(sd_primary.v, Q_INDE_PRI) *
                    IndePrimaryGrowthRate(dev_age,length)
                    +
                    2.0 * M_PI * radius * length * INTERNODE_DENSITY *
                    SinkUnloading(sd_primary.v, Q_INDE_SEC) *
                    IndeSecondaryGrowthRate(dev_age, radius));

  // for internode maintenance, demand depends on unloading and total carbon
  demand = IndeMaintenance(sd_primary.q,order) * 
           dSinkUnloading_dc(sd_maintenance.v, Q_INDE_MAN);

  sd_maintenance.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_maintenance.cp.e = sd_maintenance.v - sd_maintenance.cp.r *
                        IndeMaintenance(sd_primary.q,order) *
                        SinkUnloading(sd_maintenance.v, Q_INDE_MAN);

  // for internode reserves, calculate starch synthesis and hydrolysis

  // first, rate of flow of carbos into reserves
  demand = IndeReserveSynthesis (sd_primary.q, sd_reserves.q, INDE_SYNTHESIS) * 
           dSinkUnloading_dc(sd_reserves.v, Q_RESERVES);
  sd_reserves.cp.r = (fabs(demand) > EPSILON) ? 1.0/(demand) : INFTY;
  sd_reserves.cp.e = sd_reserves.v - sd_reserves.cp.r *
                     IndeReserveSynthesis (sd_primary.q, sd_reserves.q, INDE_SYNTHESIS) *
                     SinkUnloading(sd_reserves.v, Q_RESERVES);

  // notice, hydrolysis demands on amount of carbon in reserves (sd_reserves.q)
  demand = IndeReserveHydrolysis(sd_reserves.q, INDE_HYDROLYSIS) *
           dSourceLoading_dc(sd_hydrolysis.v, Q_INDE_SRC);
  sd_hydrolysis.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_hydrolysis.cp.e = sd_hydrolysis.v - sd_hydrolysis.cp.r *
                       IndeReserveHydrolysis(sd_reserves.q, INDE_HYDROLYSIS) *
                       SourceLoading(sd_hydrolysis.v, Q_INDE_SRC);

  produce Internode(node,age,order,dev_age,length,radius) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis);
}

Leaf(node,age,order,dev_age,area) S(sd_primary) S(sd_maintenance) S(sd_source):
{
  float demand; // the sink's carbon demand, linearized! (conductance in electrical circuit)

  // demand for growth depends on unloading function and relative growth rate
  demand = LeafPotentialGrowthRate(sd_primary.q, dev_age, node, order) *
           dSinkUnloading_dc(sd_primary.v, Q_LEAF_PRI);

  sd_primary.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_primary.cp.e = sd_primary.v - sd_primary.cp.r *
                    LeafPotentialGrowthRate(sd_primary.q, dev_age, node, order) *
                    SinkUnloading(sd_primary.v, Q_LEAF_PRI);

  // leaf maintenance depends on primary carbon
  demand = LeafMaintenance (sd_primary.q) *
           dSinkUnloading_dc(sd_maintenance.v, Q_LEAF_MAN);

  sd_maintenance.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_maintenance.cp.e = sd_maintenance.v - sd_maintenance.cp.r *
                        LeafMaintenance (sd_primary.q) *
                        SinkUnloading(sd_maintenance.v, Q_LEAF_MAN);

  // leaf output depends on carbon acquistion
  demand = LeafCarbonOutput(sd_source.q) * dSourceLoading_dc(sd_source.v, Q_LEAF_SRC);
  sd_source.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_source.cp.e = sd_source.v - sd_source.cp.r *
                                 LeafCarbonOutput(sd_source.q) *
                                 SourceLoading(sd_source.v, Q_LEAF_SRC);

  // convert carbon to leaf area
  area = Carb2Area(sd_primary.q);

  produce Leaf(node,age,order,dev_age,area) S(sd_primary) S(sd_maintenance) S(sd_source);
}

Flower(node,age,diameter) S(sd_primary):
{
  float demand; // The linearized carbon demand 

  demand = FlowerGrowthRate(INITIAL_FRUIT_CARBON) *
           dSinkUnloading_dc(sd_primary.v, Q_FLOWER);

  sd_primary.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_primary.cp.e = sd_primary.v - sd_primary.cp.r *
                    FlowerGrowthRate(INITIAL_FRUIT_CARBON) *
                    SinkUnloading(sd_primary.v, Q_FLOWER);

  diameter = sd_primary.q * FLOWER_DENSITY;

  produce Flower(node,age,diameter) S(sd_primary);
}

Fruit(node,age,volume) S(sd_primary) S(sd_maintenance):
{
  float demand; // The linearized carbon demand

  demand = FruitGrowthRate(sd_primary.q) * 
           dSinkUnloading_dc(sd_primary.v, Q_FRUIT_PRI);

  sd_primary.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_primary.cp.e = sd_primary.v - sd_primary.cp.r *
                    FruitGrowthRate(sd_primary.q) *
                    SinkUnloading(sd_primary.v, Q_FRUIT_PRI);

  demand = FruitMaintenance(sd_primary.q) * 
           dSinkUnloading_dc(sd_maintenance.v, Q_FRUIT_MAN);

  sd_maintenance.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_maintenance.cp.e = sd_maintenance.v - sd_maintenance.cp.r *
                        FruitMaintenance(sd_primary.q) *
                        SinkUnloading(sd_maintenance.v, Q_FRUIT_MAN);
  
  volume = sd_primary.q * FRUIT_DENSITY;

  produce Fruit(node,age,volume) S(sd_primary) S(sd_maintenance);
}

S(sd):
{
  // if r == INFTY this is an inert S between branches
  if (sd.cp.r != INFTY)
    Printf ("PANIC! Orphan S() module found. Check 'UpdateDemand' phase.\n");
  produce S(sd);
}

/* -------------------------------------------------------------------------- */

group Fold: // folding step, reduce neigbouring sinks into one, starting from end

consider: S;

// calculate equivalent of the circuit to the right
S(sd) >> SB() S(sdr2) EB() S(sdr1):
{
  // add serial resistances in both branches
  sdr1.ceq.r += sdr1.rs;
  sdr2.ceq.r += sdr2.rs;

  // Find the circuit equivalent to branches connected in parallel
  LinearData cr = in_parallel(sdr1.ceq, sdr2.ceq);

  // connect in parallel to the current (float) parallel circuit
  sd.ceq = in_parallel(sd.cp, cr);

  produce S(sd);
}

S(sd) >> SB() S(sdr) EB():
{
  // add serial resistance
  sdr.ceq.r += sdr.rs;

  // connect in parallel right equivalent circuit to the current (float) parallel circuit
  sd.ceq = in_parallel(sd.cp, sdr.ceq);

  produce S(sd);
}


S(sd) >> S(sdr):
{
  // add serial resistance
  sdr.ceq.r += sdr.rs;

  // connect in parallel right equivalent circuit to the current (float) parallel circuit
  sd.ceq = in_parallel(sd.cp, sdr.ceq);

  produce S(sd);
}

S(sd):
{
  sd.ceq = sd.cp;
  produce S(sd);
}

/* -------------------------------------------------------------------------- */

group Unfold: // unfolding step, find the concentration of carbon along transport pathway

consider: S;

// calculate the voltages; estimate the error
S(sdl) << S(sd):
{
  // Find the circuit representing the basal part
  LinearData cl = {sdl.v, sd.rs};

  // connect with the equivalent of the overhanging part
  sd.v = in_parallel(cl,sd.ceq).e;

  produce S(sd);
}

S(sd):
{
  sd.v = sd.ceq.e;
  produce S(sd);
}


/* -------------------------------------------------------------------------- */

group UpdateFlow: // update the flow of carbon into each source/sink

S(sd) :
{	
  extern float Serror;
  float new_i, divisor;

  if (sd.v >= 0.0)
  {
    // Calculate new current through cp
    divisor = (fabs(sd.cp.r) > EPSILON) ? sd.cp.r : EPSILON;  
    new_i = (sd.v - sd.cp.e)/divisor;

    // Calculate error and save for display purposes
    sd.err = new_i - sd.i;

    // Calculate cumulative error to determine whether it is possible to stop
    Serror += fabs(sd.err);

    // Update current to be consistent with present values of v, q
    // check for minimum current
    if (fabs(new_i) < EPSILON)
      sd.i = 0.0;
    else
      sd.i = new_i;
  }
  else // v < 0! Reset the Newton-Raphson method, with v as zero
  {
    sd.v = 0.0;
    sd.err = 1.0;
    Serror += 1.0;
  }

  produce S(sd);
}

/* -------------------------------------------------------------------------- */

group Debug: // if error doesn't fall below MAXERROR after many iterations, output debug info

S(sd):
{
  if (sd.err > MAXERROR)
    Printf ("q:%g i:%g v:%g err:%g\n", sd.q, sd.i, sd.v, sd.err);

  produce S(sd);
}


/* -------------------------------------------------------------------------- */

/* Biomechanics Rules ------------------------------------------------------- */

group RelaxFold:

consider: B;

B(id) >> SB() EB() SB() B(idb) EB() B(idr):
{
  id.sigma_mass = id.mass + idb.sigma_mass + idr.sigma_mass;

  // compute torque due to gravity
  id.torque = idb.torque + (idb.H*idb.s % (id.sigma_mass*Gravity)).z
            + idr.torque + (idr.H*idr.s % (id.sigma_mass*Gravity)).z;

  produce B(id);
}

B(id) >> SB() EB() SB() B(idb) EB():
{
  id.sigma_mass = id.mass + idb.sigma_mass;

  // compute torque due to gravity
  id.torque = idb.torque + (idb.H*idb.s % (id.sigma_mass*Gravity)).z;

  produce B(id);
}

B(id) >> B(idr):
{
  id.sigma_mass = id.mass + idr.sigma_mass;

  // compute torque due to gravity
  id.torque = idr.torque + (idr.H*idr.s % (id.sigma_mass*Gravity)).z;

  produce B(id);
}

B(id): 
{
  id.torque = 0.0;
  id.sigma_mass = id.mass;

  produce B(id);
}

/* -------------------------------------------------------------------------- */

group RelaxUnfold:

consider: B;

B(idl) << B(id):
{
  // Added branching angle 'id.ba' to the following line:
  V3f NewEquilVector = VecRotate(idl.H, id.torque * id.inv_ke + id.ba);
  V3f DifferenceVector = NewEquilVector - id.H;

  BMerror += DifferenceVector.Length();
  id.H = Normalize(id.H + BM_RELAX*DifferenceVector);

  id.P = idl.P + id.s*id.H;

  produce B(id);
}

B(id):
{
  id.P = id.s*id.H;

  produce B(id);
}

/* -------------------------------------------------------------------------- */

/* Default Group ------------------------------------------------------------ */

group 0:

// the following rule handles a pruning cut made by clicking on the shoot
MouseIns(): 
{
  // Note: this rule is in group 0, so that the mouse click is registered
  // within any of the L+C groups (phases), however, it should not produce
  // a Cut() because that might change the structure within a folding/unfolding step.
  // Even though such a change would not cause the model to stop working,
  // it may produce undesired effects (but I didn't see any in testing).
  // Thus, I think it is safer to produce a Cut() module in the 'Develop' phase.
  produce Prune();
}

/* -------------------------------------------------------------------------- */

/* Visualisation ------------------------------------------------------------ */

interpretation:

vgroup KiwifruitShoot:

consider: B;

Apex(node,age,physio_age,vigour,order):
{
  produce SetColor(15) Sphere(INITIAL_INDE_RADIUS);
}

Internode(node,age,order,dev_age,length,radius) > B(bm):
{
  static float last_radius;
  int colour = age >= 365 ? 11 : 15;  // mature internodes are brown; young are green

  produce SetColor(colour) SetWidth(radius*2.0) LineTo3f(bm.P);
}


Leaf(node,age,order,dev_age,area):
{
  float radius = sqrtf(area/M_PI); // for visualisation as a disk (area=pi*radius^2)
  produce SetColor(14)       
          SetElasticity(1,0.2)
          Down(INCLINATION_ANGLE)
          // leaf stalk
          SetWidth(radius*0.05)
          F(radius*0.25) F(radius*0.25) F(radius*0.25) F(radius*0.25)
          RollToVert() // set the turtle's up vector to point up
          // leaf
          Down(90) RollL(180)
          SB() Surface(0,radius*0.8) EB()
          SetElasticity(1,0.0);
}

Flower(node,age,diameter):
{
  // peduncle length is 10 to 40 mm
  // flower diameter is 35 to 68 mm

  float radius = diameter*0.5f;

  produce SB() RollL(PHYLLOTAXIS*node) // FOR 2D BIOMECHANICS: remove otherwise
               //Peduncle
               Down(110) SetColor(15) SetWidth(diameter*0.05) F(diameter*0.6)
               //Petals
               Down(90) SetColor(12) Circle(radius)
               //stamen (note: no stigma)
               Up(90) f(diameter*0.01) Down(90) SetColor(5) Circle(radius*0.25)
          EB();
}

Fruit(node,age,volume):
{
  float radius, height;
  float final_volume = MAX_FRUIT_CARBON * FRUIT_DENSITY;

  // max length of the fruit is 6.5cm
  // max circumference is 18cm => max radius is 2.86cm

  radius = pow(0.44*volume*M_PI,0.3333);
  height = 2.2727 * radius;

  produce SB() RollL(PHYLLOTAXIS*node) // FOR 2D BIOMECHANICS: remove otherwise
               SetElasticity(0,0.25*volume/final_volume)
               Down(110) SetColor(15) SetWidth(0.25f)
               // keep peduncle from flower (6.8 cm)
               F(6.8*0.5*0.25)
               F(6.8*0.5*0.25)
               F(6.8*0.5*0.25)
               F(6.8*0.5*0.25)
               SetColor(13) f(radius) Sphere(radius)
                            SetWidth(radius*2.0) F(height-2.0*radius)
                            Sphere(radius)
               SetElasticity(0,0.0)
          EB();
}

/* -------------------------------------------------------------------------- */

vgroup CarbonDynamics:
maximum depth: 2;

module DrawS(int);

Apex(node,age,physio_age,vigour,order) S(sd_apex):
{
  produce SetColor(5) Triangle(2.0,2.0) DrawS(5);
}

Leaf(node,age,order,dev_age,area) S(sd_primary) S(sd_maintenance) S(sd_source):
{
  produce SB() f(7.25) // move left
               Left(90) f(6.5) Left(110) SetWidth(0.1) f(0.25) SetColor(3) F(10.5) // draw line
          EB()
          f(-8.5) Right(70) f(-8)
          // draw primary sink
          SetColor(3) Circle(1.0) DrawS(3)
          // draw maintenance sink
          SB f(-1.0) Rhombus(2.0,2.0) EB DrawS(3)
          // draw source
          SB f(0.1) Right(45) f(-1.25) Rhombus(2.5,2.5) EB DrawS(3);
}

Internode(node,age,order,dev_age,length,radius) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis):
{
  // set turtle's heading, so all the rotations can be ignored
  nproduce SetHead(0,1,0,0,0,1);

  if (order != CANE_ORDER)
  {
    nproduce Left(90);

    if (order == AX_SHOOT_ORDER)
    {
      if (node == 1)
        nproduce f(-0.75) Right(90) f(13.75) Left(90);
    }
    else
    {
      for (int i = AX_SHOOT_ORDER; i < order; i++)
        nproduce Right(INCLINATION_ANGLE);
    }
  }

  produce // draw primary sink
          SetColor(4) Circle(1.0) DrawS(4)
          // draw maintenance sink
          SB f(-1.0) Rhombus(2.0,2.0) EB DrawS(4)
          // draw reserves
          SB f(-0.25) // draw a pentagon, with internal angle = 108
            SP f(1.25) PP Right(180-108*0.5)
               f(1.25) PP Right(180-108)
               f(1.25) PP Right(180-108)
               f(1.25) PP Right(180-108)
               f(1.25) PP
            EP
          EB
          DrawS(4);
}

AxBud(node,age,vigour,order) S(sd_primary):
{
  if (order == 0)
  {
    produce SetHead(0,1,0,0,0,1)
            // draw primary sink
            SetColor(5) Triangle(2.0,2.0) DrawS(5);

  }
  else
  {
    produce RollToVert() RollL(90)
            SB() Left(90) f(6.5) Right(90) // move down
                 f(7.25) // move left
                 Right(150) SetWidth(0.1) f(0.25) SetColor(5) F(11.25)  // draw line
            EB()
            f(-9.5) Right(60) f(-4.5)
            // draw primary sink
            SetColor(5) Triangle(2.0,2.0) DrawS(5);
  }
}

Flower(node,age,diameter) S(sd_primary):
{
  produce RollToVert() RollL(90)
          SB() Left(90) f(6.5) Right(90) // move down
               f(7.25) // move left
               Right(150) SetWidth(0.1) f(0.25) SetColor(12) F(11.25)  // draw line
          EB()
          f(-9.5) Right(60) f(-4.5)
          // draw primary sink
          SetColor(12) Circle(1.0) DrawS(12);
}

Fruit(node,age,volume) S(sd_primary) S(sd_maintenance):
{
  produce RollToVert() RollL(90)
          SB() Left(90) f(6.5) Right(90) // move down
               f(7.25) // move left
               Right(150) SetWidth(0.1) f(0.25) SetColor(2) F(11.25)  // draw line
          EB()
          f(-9.5) Right(60) f(-4.5)
          // draw primary sink
          SetColor(2) Circle(1.0) DrawS(2)
          // draw maintenance sink
          SB f(-1.0) Rhombus(2.0,2.0) EB DrawS(2);
}

S(sd) > SB(): // draw the inert circuit between branches
{
  if (sd.cp.r == INFTY)
  {
    nproduce SetColor(7)
             SB f(-1.) // draw a (star) pentagram, with internal angle = 36
                SB SP f(2.) PP Right(180-36*0.5)
                   f(2.) PP Right(180-36)
                   f(1.) PP
                EP EB
                SB SP f(2.) PP Left(180-36*0.5)
                   f(2.) PP Left(180-36)
                   f(1.) PP
                EP EB
                SB
                  f(2.-1.*tan(0.62831)) TurnAround() Triangle(2.,1.*tan(0.62831))
                EB
             EB
             DrawS(7);
  }
  produce;
}

DrawS(colour):
{
  produce SB()
          f(1.0) SetWidth(0.1) F(2.0)
          SetWidth(1) F(2.0)                                           // resistance in parallel
          SetWidth(0.1) F(2.0)
          f(0.25) SetColor(1) Circle(0.25)                             // voltage
          Left(90) f(0.25) SetColor(colour) SetWidth(0.1) F(2.0)
          SetWidth(1) F(2.0)                                           // resistance in series
          SetWidth(0.1) F(2.0)
          EB() Right(90) f(6.5) Left(90);

}

/* -------------------------------------------------------------------------- */

vgroup AuxinFlow:

Internode(node,age,order,dev_age,length,radius):
{
  if (order != CANE_ORDER)
    nproduce SetHead(0,1,0,0,0,1);

  if (order > AX_SHOOT_ORDER)
  {
    for (int i = AX_SHOOT_ORDER; i < order; i++)
      nproduce Right(INCLINATION_ANGLE);
    if (node == 1)
      nproduce f(H*0.5);
  }

  produce f(H*1.5);
}

AxBud(node,age,vigour,order):
{
  nproduce SetHead(0,1,0,0,0,1);

  if (order > AX_SHOOT_ORDER)
  {
    for (int i = AX_SHOOT_ORDER; i < order; i++)
      nproduce Right(INCLINATION_ANGLE);
  }
  
  produce f(H) Right(90) f(H*1.5);
}

Apex(node,age,physio_age,vigour,order):
{
  nproduce SetHead(0,1,0,0,0,1);
  if (order > AX_SHOOT_ORDER)
  {
    for (int i = AX_SHOOT_ORDER; i < order; i++)
      nproduce Right(INCLINATION_ANGLE);
  }
  produce f(H*1.5);
}

W(d) A(s):
{
  float c, flux, PINi;
  int colour;
  if (d.flux >= 0.0)
    colour = 3;
  else
    colour = 4;

  // scale values for visualisation
  c = s.c*0.5;
  flux = d.flux*0.5;
  PINi = d.PINi*0.5;

  produce // draw a square
	  SetColor(1)
          SB
            SP f(H) PP Right(90)
               f(H) PP Right(90)
               f(H) PP Right(90)
               f(H) PP
            EP
          EB
          // visualize auxin concentration
          SB SetColor(7)
            Up(90) f(0.1) Down(90) // move away from black square
            Right(45) f(0.707107*(H-c)) Left(45) // move to centre
            SP f(c) PP Right(90)
               f(c) PP Right(90)
               f(c) PP Right(90)
               f(c) PP
            EP
          EB
          // visualize PIN concentration
          SB SetColor(5)
            Right(90) f(H*0.5) Left(90)
            SetWidth(H)
            F(-PINi)
          EB
          // visualize auxin efflux
          SB
            Right(90) f(H*0.5) Left(90)
            SetColor(colour) SetWidth(fabs(flux)) F(-H*0.5)
          EB
          ;
}

A(s) > W(r):  // this is the Root module
{
  float c;

  c = s.c * 0.5;

  produce SetHead(1,0,0,0,0,1)
          // draw a square
	  SetColor(2)
          SB
            SP f(H) PP Right(90)
               f(H) PP Right(90)
               f(H) PP Right(90)
               f(H) PP
            EP
          EB
          // visualize auxin concentration
          SB SetColor(7)
            Up(90) f(0.1) Down(90) // move away from black square
            Right(45) f(0.707107*(H-c)) Left(45) // move to centre
            SP f(c) PP Right(90)
               f(c) PP Right(90)
               f(c) PP Right(90)
               f(c) PP
            EP
          EB
          ;
}

/* -------------------------------------------------------------------------- */

vgroup Biomechanics:

B(id) :
{
  // the original model used position of the node for drawing, as follows
  // produce SetColor(5) SetWidth(0.75) LineTo3f(id.P) SetColor(1) Sphere(id.mass*2.5);
  // but it is also possible to use the orientation directly, because this is in 2D.
  produce SetHead(id.H.x,id.H.y,id.H.z,0,0,1)
          SetColor(5) SetWidth(0.75) F(id.s)
          SetColor(1) f(id.mass) Sphere(id.mass*2.5) f(id.mass);
}

/* -------------------------------------------------------------------------- */

